﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Security.RightsManagement;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using System.Diagnostics;

namespace ColorPicker.ViewModel
{
    public partial class ColorPickerViewModel : ObservableObject
    {
        public ColorPickerViewModel()
        {
            red = 255;
            green = 0;
            blue = 0;
            CalculateHex();
        }

        [ObservableProperty]
        byte red;

        [ObservableProperty]
        byte green;

        [ObservableProperty]
        byte blue;

        [ObservableProperty]
        string hex;

        [ObservableProperty]
        string hexInput;

        bool lockEvents = false;

        partial void OnRedChanged(byte value)
        {
            if (lockEvents) return;
            CalculateHex();
        }

        partial void OnGreenChanged(byte value)
        {
            if (lockEvents) return;
            CalculateHex();
        }

        partial void OnBlueChanged(byte value)
        {
            if (lockEvents) return;
            CalculateHex();
        }

        partial void OnHexInputChanged(string value)
        {
            if (lockEvents || !Regex.IsMatch(value, @"^#[\dA-F]{6}$", RegexOptions.IgnoreCase))
                return;

            Hex = HexInput;
            CalculateRGB();
        }

        public void CalculateHex()
        {
            Hex = "#" + Red.ToString("X2") + Green.ToString("X2") + Blue.ToString("X2");
            HexInput = Hex;
        }

        public void CalculateRGB()
        {
            lockEvents = true;
            Red = byte.Parse(Hex.Substring(1, 2), System.Globalization.NumberStyles.HexNumber);
            Green = byte.Parse(Hex.Substring(3, 2), System.Globalization.NumberStyles.HexNumber);
            Blue = byte.Parse(Hex.Substring(5, 2), System.Globalization.NumberStyles.HexNumber);
            lockEvents = false;
        }
    }
}
